@@ -1,9 +1,10 @@
 import csv
+from daspython.common.response import HasEntry, HasItems, HasTotal
 import uuid
 import json
 from pathlib import Path
 from requests.api import request
-from daspython.common.api import ApiMethods, Token
+from daspython.common.api import ApiMethods, Response, Token
 from daspython.services.attributes.attributeservice import AttributeService
 from daspython.services.searches.searchservice import SearchEntriesRequest, SearchService
 from daspython.services.entryfields.entryfieldservice import DisplayType, EntryFieldService
@@ -128,12 +129,14 @@ class GetEntryRelationsRequest():
     relationtype: int = 1
     deeplevel: int = 1
 
+class GetEntryResponse(HasEntry):
+    pass
 
 class EntryService(ApiMethods):
     def __init__(self, auth: Token):
         super().__init__(auth)
 
-    def get_all(self, request: GetAllEntriesRequest):
+    def get_all(self, request: GetAllEntriesRequest) -> Response:
         '''
         Get all entries based on the request parameter values.
 
@@ -147,9 +150,10 @@ class EntryService(ApiMethods):
             A json that represents  a list of entries.                           
         '''
         api_url = '/api/services/app/Entry/GetAll?'
-        return self.get_data(url=api_url, request=request)
+        response = self.get_data(url=api_url, request=request)
+        return super()._get_entries_response(response)
 
-    def get_entry_by_code(self, code: str):
+    def get_entry_by_code(self, code: str) -> GetEntryResponse:
         '''
         Get an entry by its code.
 
@@ -165,7 +169,50 @@ class EntryService(ApiMethods):
         api_url = '/api/services/app/Entry/GetEntryByCode?'
         request = GetEntryByCodeRequest()
         request.code = code
-        return self.get_data(url=api_url, request=request)
+        response =  self.get_data(url=api_url, request=request)
+        
+        if (response == None):
+            return None
+
+        if (response.get('result') == None):
+            return None
+
+        result = GetEntryResponse()            
+        result.entry = response['result'].get('entry')
+        return result
+
+
+    def get_entry_by_name(self, name: str, attribute_name: str):
+        '''
+        Get an entry by its name.
+
+        Parameters
+        ----------
+        name : str
+            An given name from an entry.
+
+        Returns
+        -------
+            A json that represents  an entry.                           
+        '''
+        search = SearchService(self.token)
+        request = SearchEntriesRequest()
+
+        attribute = AttributeService(self.token)
+        attribute_id = attribute.get_attribute_id(attribute_name)
+
+        request.attributeId = attribute_id
+        request.querystring = f'displayname({name})'
+
+        response = search.search_entries(request)
+
+        if (response == None):
+            return None
+
+        if (response.total == 0):
+            return None
+
+        return response.items[0]
 
     def get(self, request: GetEntryRequest):
         '''
@@ -214,7 +261,8 @@ class EntryService(ApiMethods):
         '''
         body.entry['id'] = str(uuid.uuid1())
         api_url = '/api/services/app/Entry/Create'
-        return self.post_data(url=api_url, body=body)
+        response =  self.post_data(url=api_url, body=body)
+        return response['result']
 
     def update(self, body: UpdateRequest):
         '''
@@ -230,7 +278,8 @@ class EntryService(ApiMethods):
             The updated entry's indentifier.
         '''
         api_url = '/api/services/app/Entry/Update'
-        return self.put_data(url=api_url, body=body)
+        response = self.put_data(url=api_url, body=body)
+        return response['result']
 
     def delete_entry(self, id, attributeId):
         '''
@@ -247,9 +296,10 @@ class EntryService(ApiMethods):
             True if the entry was deleted successfully.
         '''
         api_url = f'/api/services/app/Entry/Delete?Id={id}&AttributeId={attributeId}'
-        return self.delete_data(url=api_url)
+        response = self.delete_data(url=api_url)
+        return response['result']
 
-    def get_entry_id(self, name: str, attribute_name: str = None, attribute_id: int = None ) -> str:
+    def get_entry_id(self, name: str, attribute_name: str = None, attribute_id: int = None) -> str:
         '''
         Gets the entry identifier based on its name and attribute.
 
@@ -279,23 +329,24 @@ class EntryService(ApiMethods):
         request.attributeId = attribute_id
         response = search.search_entries(request)
 
-        if (response == None or response['result'] == None or response['result']['totalCount'] == None or response['result']['totalCount'] == 0):
+        if (response == None or response.total == 0):
             return None
 
-        return response['result']['items'][0]['entry']['id']
+        return response.items[0]['id']
 
-    def get_csv_file(self, attributeName: str, file_name:str = None):
+    def create_csv_template(self, attributeName: str, file_name: str = None):
 
         service = AttributeService(self.token)
         id = service.get_attribute_id(attributeName)
 
-        path = Path(f'{attributeName if None == file_name else file_name }.csv')
+        path = Path(
+            f'{attributeName if None == file_name else file_name }.csv')
 
         service = EntryFieldService(self.token)
         response = service.get_all(id, DisplayType.FORM)
 
         customdata_list = list(
-            map(lambda kv: (kv['column'], kv['customData']), response['result']['items']))
+            map(lambda kv: (kv['column'], kv['customData']), response.items))
 
         remove_list = list(
             filter(lambda x: (x[1] != None and 'readonly' in x[1]), customdata_list))
@@ -303,7 +354,8 @@ class EntryService(ApiMethods):
         remove_list.append('6')
         remove_list.append('code')
 
-        header_list = sorted(response['result']['items'], key=lambda field: field['y'])
+        header_list = sorted(
+            response.items, key=lambda field: field['y'])
         header_list = list(map(lambda kv: kv['column'], header_list))
 
         header_list = list(
@@ -315,7 +367,7 @@ class EntryService(ApiMethods):
         service = EntryFieldService(self.token)
         fields = service.get_all(attributeId, displayType)
         customdata_list = list(
-            map(lambda kv: (kv['column'], kv['customData']), fields['result']['items']))
+            map(lambda kv: (kv['column'], kv['customData']), fields.items))
         customdata_list = list(
             filter(lambda x: (x[1] != None), customdata_list))
         return customdata_list
@@ -329,16 +381,17 @@ class EntryService(ApiMethods):
                 datasource = json.loads(customdata[1])
                 if ('attributeId' in datasource['datasource']):
                     attribute_id = datasource['datasource']['attributeId']
-                    entry[customdata[0]] = service.get_entry_id(entry[customdata[0]], attribute_id=attribute_id)
+                    entry[customdata[0]] = service.get_entry_id(
+                        entry[customdata[0]], attribute_id=attribute_id)
 
-        return entry   
+        return entry
 
     def _get_entry(self, header, row) -> any:
         entry = {}
         for index, h in enumerate(header):
             entry[h] = row[index]
 
-        return entry             
+        return entry
 
     def insert_entries_from_csv(self, csv_file: str, attribute: str):
 
@@ -360,9 +413,10 @@ class EntryService(ApiMethods):
                 row_num += 1
 
         service = AttributeService(self.token)
-        attribute_id =  service.get_attribute_id(attribute)               
+        attribute_id = service.get_attribute_id(attribute)
 
-        customdata_fields = self._get_customdata_fields(attribute_id, DisplayType.FORM)
+        customdata_fields = self._get_customdata_fields(
+            attribute_id, DisplayType.FORM)
 
         for entry in new_entries:
             entry = self._get_customdata_values(customdata_fields, entry)
@@ -370,6 +424,3 @@ class EntryService(ApiMethods):
             request = InsertRequest()
             request.entry = entry
             self.create(request)
-
-                          
-
